/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.config.tablesection;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.lang3.ArrayUtils;
import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.config.tablesection.alignments.ColumnAlign;
import org.dussan.vaadin.dtable.config.tablesection.alignments.ColumnAlignment;
import org.dussan.vaadin.dtable.config.tablesection.alignments.RowAlign;
import org.dussan.vaadin.dtable.config.tablesection.alignments.RowAlignment;
import org.dussan.vaadin.dtable.config.tablesection.dimensions.RowHeight;
import org.dussan.vaadin.dtable.config.tablesection.margins.Margin;

public class TableSectionConfig
    implements Serializable, RowHeight, ColumnAlignment, RowAlignment, Margin
{

    private static final long serialVersionUID = 5594381047713674462L;

    private boolean verticalLinesEnabled = false;

    private boolean expandedFrozenCellEnabled = false;

    private boolean expandedFrozenCellAlwaysVisibleEnabled = false;

    private boolean expandedCellEnabled = false;

    private boolean expandedCellAlwaysVisibleEnabled = false;

    private boolean wordWrapEnabled = false;

    private boolean checkColumnsAlignment = true;

    private int allColumns = 0;

    private int rowHeight = RowHeight.ROW_DEFAULT_HEIGHT;

    private int lineHeight = RowHeight.LINE_DEFAULT_HEIGHT;

    private int topMargin = 0;

    private int rightMargin = 0;

    private int bottomMargin = 0;

    private int leftMargin = 0;

    private Map<Integer, ColumnAlign> columnsAlignment = new HashMap<>();

    private ColumnAlign columnDefaultAlignment = ColumnAlign.LEFT;

    private RowAlign rowAlignment = RowAlign.MIDDLE;

    public void setAllColumns( int allColumns )
    {
        this.allColumns = allColumns;
    }

    private boolean isHeightOK( int height )
    {
        return 0 < height;
    }

    public boolean isVerticalLinesEnabled()
    {
        return verticalLinesEnabled;
    }

    public void setVerticalLinesEnabled( boolean verticalLinesEnabled )
    {
        this.verticalLinesEnabled = verticalLinesEnabled;
    }

    public boolean isExpandedFrozenCellEnabled()
    {
        return expandedFrozenCellEnabled;
    }

    public void setExpandedFrozenCellEnabled( boolean expandedFrozenCellEnabled )
    {
        this.expandedFrozenCellEnabled = expandedFrozenCellEnabled;
    }

    public void setExpandedFrozenCellEnabled( boolean expandedFrozenCellEnabled,
                                              boolean expandedFrozenCellAlwaysVisibleEnabled )
    {
        this.expandedFrozenCellEnabled = expandedFrozenCellEnabled;
        this.expandedFrozenCellAlwaysVisibleEnabled = expandedFrozenCellAlwaysVisibleEnabled;
    }

    public boolean isExpandedFrozenCellAlwaysVisibleEnabled()
    {
        return expandedFrozenCellAlwaysVisibleEnabled;
    }

    public void setExpandedFrozenCellAlwaysVisibleEnabled( boolean expandedFrozenCellAlwaysVisibleEnabled )
    {
        this.expandedFrozenCellAlwaysVisibleEnabled = expandedFrozenCellAlwaysVisibleEnabled;
    }

    public boolean isExpandedCellEnabled()
    {
        return expandedCellEnabled;
    }

    public void setExpandedCellEnabled( boolean expandedCellEnabled )
    {
        this.expandedCellEnabled = expandedCellEnabled;
    }

    public void setExpandedCellEnabled( boolean expandedCellEnabled, boolean expandedCellAlwaysVisibleEnabled )
    {
        this.expandedCellEnabled = expandedCellEnabled;
        this.expandedCellAlwaysVisibleEnabled = expandedCellAlwaysVisibleEnabled;
    }

    public boolean isExpandedCellAlwaysVisibleEnabled()
    {
        return expandedCellAlwaysVisibleEnabled;
    }

    public void setExpandedCellAlwaysVisibleEnabled( boolean expandedCellAlwaysVisibleEnabled )
    {
        this.expandedCellAlwaysVisibleEnabled = expandedCellAlwaysVisibleEnabled;
    }

    public boolean isWordWrapEnabled()
    {
        return wordWrapEnabled;
    }

    public void setWordWrapEnabled( boolean wordWrapEnabled )
    {
        this.wordWrapEnabled = wordWrapEnabled;
    }

    @Override
    public int getRowHeight()
    {
        return rowHeight;
    }

    @Override
    public void setRowHeight( int height )
    {
        rowHeight = isHeightOK( height ) ? height : rowHeight;
    }

    @Override
    public int getLineHeight()
    {
        return lineHeight;
    }

    @Override
    public void setLineHeight( int height )
    {
        lineHeight = isHeightOK( height ) ? height : lineHeight;
    }

    @Override
    public ColumnAlign getDefaultColumnAlignment()
    {
        return columnDefaultAlignment;
    }

    @Override
    public void setDefaultColumnAlignment( ColumnAlign alignment )
    {
        columnDefaultAlignment = alignment != null ? alignment : columnDefaultAlignment;
    }

    private void checkColumnsAlignment()
    {
        if ( checkColumnsAlignment && 0 < allColumns )
        {
            if ( !columnsAlignment.isEmpty() )
            {
                // check columns number
                List<Integer> columns = Arrays.asList( columnsAlignment.keySet()
                                                                       .toArray( new Integer[] {} ) );
                Collections.sort( columns );
                if ( !columns.isEmpty() && !( 0 <= columns.get( 0 ) && columns.get( columns.size() - 1 ) < allColumns ) )
                {
                    Map<Integer, ColumnAlign> alignments = new HashMap<>();
                    for ( int column : columns )
                    {
                        if ( 0 <= column && column < allColumns )
                        {
                            alignments.put( column, columnsAlignment.get( column ) );
                        }
                    }
                    columnsAlignment = new HashMap<>( alignments );
                }
            }

            // add missing columns width
            if ( columnsAlignment.size() < allColumns )
            {
                for ( int column = 0; column < allColumns; column++ )
                {
                    if ( !columnsAlignment.containsKey( column ) )
                    {
                        columnsAlignment.put( column, columnDefaultAlignment );
                    }
                }
            }
            checkColumnsAlignment = false;
        }
    }

    @Override
    public ColumnAlign getColumnAlignment( int column )
    {
        try
        {
            columnsAlignment.get( column );
        }
        catch ( Exception e )
        {
            Logger.getLogger( DTable.class.getName() )
                  .log( Level.INFO, e.getLocalizedMessage(), e );
        }
        return columnDefaultAlignment;
    }

    @Override
    public void setColumnAlignment( int column, ColumnAlign columnAlign )
    {
        checkColumnsAlignment = true;
        columnsAlignment.put( column, columnAlign != null ? columnAlign : columnDefaultAlignment );
    }

    @Override
    public ColumnAlign[] getColumnsAlignment()
    {
        checkColumnsAlignment();
        ColumnAlign[] alignments = new ColumnAlign[columnsAlignment.size()];
        for ( int column = 0; column < columnsAlignment.size(); column++ )
        {
            alignments[column] = columnsAlignment.get( column );
        }
        return alignments;
    }

    @Override
    public int[] getColumnsAlignmentIntArray()
    {
        checkColumnsAlignment();
        List<Integer> alignments = new ArrayList<>();
        for ( int column = 0; column < columnsAlignment.size(); column++ )
        {
            alignments.add( columnsAlignment.get( column )
                                            .getAlignment() );
        }
        return ArrayUtils.toPrimitive( alignments.toArray( new Integer[] {} ) );
    }

    @Override
    public int[] getColumnsAlignmentIntArray( int[] columns )
    {
        checkColumnsAlignment();
        List<Integer> alignments = new ArrayList<>();
        for ( int column : columns )
        {
            alignments.add( columnsAlignment.get( column )
                                            .getAlignment() );
        }
        return ArrayUtils.toPrimitive( alignments.toArray( new Integer[] {} ) );
    }

    @Override
    public void setColumnsAlignment( ColumnAlign... alignments )
    {
        checkColumnsAlignment = true;
        for ( int column = 0; column < alignments.length; column++ )
        {
            setColumnAlignment( column, alignments[column] );
        }
    }

    @Override
    public void setColumnsAlignment( ColumnAlign alignment, int... columns )
    {
        checkColumnsAlignment = true;
        for ( int column : columns )
        {
            setColumnAlignment( column, alignment );
        }
    }

    @Override
    public void setColumnsAlignment( Map<Integer, ColumnAlign> alignments )
    {
        checkColumnsAlignment = true;
        for ( Entry<Integer, ColumnAlign> entry : alignments.entrySet() )
        {
            int column = entry.getKey();
            ColumnAlign align = entry.getValue();
            setColumnAlignment( column, align );
        }
    }

    @Override
    public RowAlign getRowAlignment()
    {
        return rowAlignment;
    }

    @Override
    public void setRowAlignment( RowAlign rowAlign )
    {
        rowAlignment = rowAlign != null ? rowAlign : rowAlignment;
    }

    @Override
    public int getTopMargin()
    {
        return topMargin;
    }

    @Override
    public int getRightMargin()
    {
        return rightMargin;
    }

    @Override
    public int getBottomMargin()
    {
        return bottomMargin;
    }

    @Override
    public int getLeftMargin()
    {
        return leftMargin;
    }

    @Override
    public void setMargins( int topMargin, int rightMargin, int bottomMargin, int leftMargin )
    {
        this.topMargin = 0 < topMargin ? topMargin : this.topMargin;
        this.rightMargin = 0 < rightMargin ? rightMargin : this.rightMargin;
        this.bottomMargin = 0 < bottomMargin ? bottomMargin : this.bottomMargin;
        this.leftMargin = 0 < leftMargin ? leftMargin : this.leftMargin;
    };

    @Override
    public void setMargins( int topBottomMargin, int leftRightMargin )
    {
        this.topMargin = 0 < topBottomMargin ? topBottomMargin : this.topMargin;
        this.rightMargin = 0 < leftRightMargin ? leftRightMargin : this.rightMargin;
        this.bottomMargin = 0 < topBottomMargin ? topBottomMargin : this.bottomMargin;
        this.leftMargin = 0 < leftRightMargin ? leftRightMargin : this.leftMargin;
    };

    @Override
    public void setTopBottomMargins( int topMargin, int bottomMargin )
    {
        this.topMargin = 0 < topMargin ? topMargin : this.topMargin;
        this.bottomMargin = 0 < bottomMargin ? bottomMargin : this.bottomMargin;
    };

    @Override
    public void setTopBottomMargins( int margin )
    {
        topMargin = 0 < margin ? margin : topMargin;
        bottomMargin = 0 < margin ? margin : bottomMargin;
    };

    @Override
    public void setLeftRightMargins( int leftMargin, int rightMargin )
    {
        this.leftMargin = 0 < leftMargin ? leftMargin : this.leftMargin;
        this.rightMargin = 0 < rightMargin ? rightMargin : this.rightMargin;
    };

    @Override
    public void setLeftRightMargins( int margin )
    {
        leftMargin = 0 < margin ? margin : leftMargin;
        rightMargin = 0 < margin ? margin : rightMargin;
    };

}
