/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.query;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.dussan.vaadin.dtable.query.elements.Field;
import org.dussan.vaadin.dtable.query.elements.Functions;
import org.dussan.vaadin.dtable.query.elements.Having;
import org.dussan.vaadin.dtable.query.elements.HavingType;
import org.dussan.vaadin.dtable.query.elements.OrderBy;
import org.dussan.vaadin.dtable.query.elements.TablesJoin;
import org.dussan.vaadin.dtable.query.elements.Where;
import org.dussan.vaadin.dtable.query.elements.WhereType;

public class Query
    implements Serializable
{

    private static final long serialVersionUID = -4764808594557072651L;

    private long startRow = 0;

    private int rows = 0;

    private String fromTable = null;

    private String fromTableAlias = null;

    private transient Map<Integer, Field> fields = null;

    private transient List<TablesJoin> tableJoins = null;

    private transient List<Object> whereCondition = null;

    private WhereType whereMainType = WhereType.AND;

    private transient List<Object> groupBy = null;

    private transient List<Object> havingCondition = null;

    private HavingType havingMainType = HavingType.OR;

    private transient List<OrderBy> orderBy = null;

    public Query()
    {
        fromTable = new String();
        fromTableAlias = new String();
        fields = new HashMap<>();
        tableJoins = new ArrayList<>();
        whereCondition = new ArrayList<>();
        groupBy = new ArrayList<>();
        havingCondition = new ArrayList<>();
        orderBy = new ArrayList<>();
    }

    private Query( long startRow, int rows, String fromTable, String fromTableAlias, Map<Integer, Field> fields,
                   List<TablesJoin> tableJoins, List<Object> whereCondition, WhereType whereMainMerge,
                   List<Object> groupBy, List<Object> havingCondition, HavingType havingMainMerge,
                   List<OrderBy> orderBy )
    {
        this.startRow = new Long( startRow );
        this.rows = new Integer( rows );

        // from table
        if ( fromTable != null )
        {
            this.fromTable = new String( fromTable );
        }
        else
        {
            this.fromTable = new String();
        }

        // from table alias
        if ( fromTableAlias != null )
        {
            this.fromTableAlias = new String( fromTableAlias );
        }
        else
        {
            this.fromTableAlias = new String();
        }

        // fields
        if ( fields != null )
        {
            this.fields = new HashMap<>( fields );
        }
        else
        {
            this.fields = new HashMap<>();
        }

        // table joins
        if ( tableJoins != null )
        {
            this.tableJoins = new ArrayList<>( tableJoins );
        }
        else
        {
            this.tableJoins = new ArrayList<>();
        }

        // where condition
        if ( whereCondition != null )
        {
            this.whereCondition = new ArrayList<>( whereCondition );
        }
        else
        {
            this.whereCondition = new ArrayList<>();
        }

        // where main type
        this.whereMainType = whereMainMerge;

        // group by
        if ( groupBy != null )
        {
            this.groupBy = new ArrayList<>( groupBy );
        }
        else
        {
            this.groupBy = new ArrayList<>();
        }

        // having condition
        if ( havingCondition != null )
        {
            this.havingCondition = new ArrayList<>( havingCondition );
        }
        else
        {
            this.havingCondition = new ArrayList<>();
        }

        // having main type
        this.havingMainType = havingMainMerge;

        if ( orderBy != null )
        {
            this.orderBy = new ArrayList<>( orderBy );
        }
        else
        {
            this.orderBy = new ArrayList<>();
        }
    }

    public static Query clone( Query query )
    {
        return new Query( query.startRow, query.rows, query.fromTable, query.fromTableAlias, query.fields,
                          query.tableJoins, query.whereCondition, query.whereMainType, query.groupBy,
                          query.havingCondition, query.havingMainType, query.orderBy );
    }

    public Query addLimit( long startRow, int rows )
    {
        this.startRow = startRow;
        this.rows = rows;
        return this;
    }

    public Query addFromTable( String fromTable, String fromTableAlias )
    {
        this.fromTable = fromTable;
        this.fromTableAlias = fromTableAlias;
        return this;
    }

    public Query addFromTable( String fromTable )
    {
        this.fromTable = fromTable;
        this.fromTableAlias = fromTable;
        return this;
    }

    public Query addFromTableAlias( String fromTableAlias )
    {
        this.fromTableAlias = fromTableAlias;
        return this;
    }

    public Map<Integer, Field> getFields()
    {
        return fields;
    }

    public Query resetFields()
    {
        fields.clear();
        return this;
    }

    public Query removeFields( int... columns )
    {
        if ( columns != null && 0 < columns.length )
        {
            for ( int column : columns )
            {
                fields.remove( column );
            }
        }
        return this;
    }

    public Query removeFields( String... fields )
    {
        if ( fields != null && 0 < fields.length )
        {
            for ( String field : fields )
            {
                for ( int column : this.fields.keySet() )
                {
                    if ( this.fields.get( column )
                                    .getName()
                                    .equals( field ) )
                    {
                        this.fields.remove( column );
                        break;
                    }
                }
            }
        }
        return this;
    }

    public Query addField( String name )
    {
        fields.put( fields.size(), new Field( name ) );
        return this;
    }

    public Query addField( String name, String alias )
    {
        fields.put( fields.size(), new Field( name, alias ) );
        return this;
    }

    public Query addField( String name, Functions function )
    {
        fields.put( fields.size(), new Field( name, function ) );
        return this;
    }

    public Query addField( String name, String alias, Functions function )
    {
        fields.put( fields.size(), new Field( name, alias, function ) );
        return this;
    }

    public Query addField( Field field )
    {
        fields.put( fields.size(), field );
        return this;
    }

    public Query addFields( String... names )
    {
        for ( String name : names )
        {
            addField( name );
        }
        return this;
    }

    public Query addFields( Field... fields )
    {
        for ( Field field : fields )
        {
            addField( field );
        }
        return this;
    }

    public Query addFields( Map<Integer, Field> fields )
    {
        this.fields.putAll( fields );
        return this;
    }

    public Query setField( int column, String name )
    {
        fields.put( column, new Field( name ) );
        return this;
    }

    public Query setField( int column, String name, String alias )
    {
        fields.put( column, new Field( name, alias ) );
        return this;
    }

    public Query setField( int column, String name, Functions function )
    {
        fields.put( column, new Field( name, function ) );
        return this;
    }

    public Query setField( int column, String name, String alias, Functions function )
    {
        fields.put( column, new Field( name, alias, function ) );
        return this;
    }

    public Query setField( int column, Field field )
    {
        fields.put( column, field );
        return this;
    }

    public TablesJoin[] getTableJoins()
    {
        return tableJoins.toArray( new TablesJoin[] {} );
    }

    public Query resetJoins()
    {
        tableJoins.clear();
        return this;
    }

    public Query addJoins( TablesJoin... joins )
    {
        this.tableJoins.addAll( Arrays.asList( joins ) );

        Set<TablesJoin> withoutDuplicates = new LinkedHashSet<>( this.tableJoins );
        this.tableJoins = new ArrayList<>( withoutDuplicates );

        return this;
    }

    public Query addJoin( TablesJoin join )
    {
        addJoins( join );
        return this;
    }

    private String getSqlWhereCondition()
    {
        if ( whereCondition != null && !whereCondition.isEmpty() )
        {
            switch ( whereMainType )
            {
                case AND:
                    return Where.AND( whereCondition.toArray() );
                case OR:
                    return Where.OR( whereCondition.toArray() );
            }
        }
        return null;
    }

    public Object[] getWhereCondition()
    {
        return whereCondition.toArray();
    }

    public WhereType getWhereMainType()
    {
        return whereMainType;
    }

    public Query setWhereMainType( WhereType whereMainType )
    {
        this.whereMainType = whereMainType;
        return this;
    }

    public Query resetWhereCondition()
    {
        whereCondition.clear();
        return this;
    }

    public Query addWhereCondition( Object... whereCondition )
    {
        this.whereCondition.addAll( Arrays.asList( whereCondition ) );

        Set<Object> withoutDuplicates = new LinkedHashSet<>( this.whereCondition );
        this.whereCondition = new ArrayList<>( withoutDuplicates );

        return this;
    }

    public Query addWhereCondition( Where whereCondition )
    {
        addWhereCondition( whereCondition );
        return this;
    }

    public Object[] getGroupBy()
    {
        return groupBy.toArray();
    }

    public Query resetGroupBy()
    {
        groupBy.clear();
        return this;
    }

    public Query removeGroupBy( String... fields )
    {
        if ( fields != null && 0 < fields.length )
        {
            for ( String field : fields )
            {
                for ( Object group : this.groupBy )
                {
                    if ( group.equals( field ) )
                    {
                        this.groupBy.remove( this.groupBy.indexOf( group ) );
                        break;
                    }
                }
            }
        }
        return this;
    }

    public Query addGroupBy( Object... groupBy )
    {
        this.groupBy.addAll( Arrays.asList( groupBy ) );

        Set<Object> withoutDuplicates = new LinkedHashSet<>( this.groupBy );
        this.groupBy = new ArrayList<>( withoutDuplicates );

        for ( int column : fields.keySet() )
        {
            if ( this.groupBy.contains( fields.get( column )
                                              .getAlias() )
                && !Functions.NONE.equals( fields.get( column )
                                                 .getFunction() ) )
            {
                removeGroupBy( fields.get( column )
                                     .getAlias() );
            }
        }

        return this;
    }

    public OrderBy[] getOrderBy()
    {
        return orderBy.toArray( new OrderBy[] {} );
    }

    public Query resetOrderBy()
    {
        orderBy.clear();
        return this;
    }

    public Query removeOrderBy( String... fields )
    {
        if ( fields != null && 0 < fields.length )
        {
            for ( String field : fields )
            {
                for ( OrderBy order : this.orderBy )
                {
                    if ( order.getField()
                              .equals( field ) )
                    {
                        this.orderBy.remove( this.orderBy.indexOf( order ) );
                        break;
                    }
                }
            }
        }
        return this;
    }

    public Query removeOrderBy( OrderBy... orders )
    {
        if ( orders != null && 0 < orders.length )
        {
            for ( OrderBy order : orders )
            {
                removeOrderBy( order.getField() );
            }
        }
        return this;
    }

    public Query addOrderBy( OrderBy... orderBy )
    {
        removeOrderBy( orderBy );
        this.orderBy.addAll( Arrays.asList( orderBy ) );

        Set<OrderBy> withoutDuplicates = new LinkedHashSet<>( this.orderBy );
        this.orderBy = new ArrayList<>( withoutDuplicates );

        return this;
    }

    private String getSqlHavingCondition()
    {
        if ( havingCondition != null && !havingCondition.isEmpty() )
        {
            switch ( whereMainType )
            {
                case AND:
                    return Having.AND( havingCondition.toArray() );
                case OR:
                    return Having.OR( havingCondition.toArray() );
            }
        }
        return null;
    }

    public Object[] getHavingCondition()
    {
        return havingCondition.toArray();
    }

    public HavingType getHavingMainType()
    {
        return havingMainType;
    }

    public Query setHavingMainType( HavingType havingMainType )
    {
        this.havingMainType = havingMainType;
        return this;
    }

    public Query resetHavingCondition()
    {
        havingCondition.clear();
        return this;
    }

    public Query addHavingCondition( Object... havingCondition )
    {
        this.havingCondition.addAll( Arrays.asList( havingCondition ) );

        Set<Object> withoutDuplicates = new LinkedHashSet<>( this.havingCondition );
        this.havingCondition = new ArrayList<>( withoutDuplicates );

        return this;
    }

    public Query addHavingCondition( Having havingCondition )
    {
        addHavingCondition( havingCondition );
        return this;
    }

    private String getSqlSelect()
    {
        if ( fields != null && !fields.isEmpty() )
        {
            StringBuilder selectBuilder = new StringBuilder();
            for ( int column = 0; column < fields.size(); column++ )
            {
                selectBuilder.append( "," );
                selectBuilder.append( fields.get( column )
                                            .getSqlField() );
            }
            return "SELECT " + selectBuilder.substring( ",".length() );
        }
        return null;
    }

    private String getSqlSelectFrom()
    {
        if ( !fields.isEmpty() && fromTable != null && !fromTable.isEmpty() )
        {
            return getSqlSelect() + " FROM " + fromTable + " " + fromTableAlias;
        }
        return null;
    }

    private String getSqlSelectFromWithJoins()
    {
        if ( getSqlSelectFrom() != null && tableJoins != null && !tableJoins.isEmpty() )
        {
            StringBuilder joinsBuilder = new StringBuilder();
            for ( TablesJoin join : tableJoins )
            {
                joinsBuilder.append( " " );
                joinsBuilder.append( join.getTablesJoin() );
            }
            return getSqlSelectFrom() + " " + joinsBuilder.substring( " ".length() );
        }
        return getSqlSelectFrom();
    }

    private String getSqlSelectFromWithWhereCondition()
    {
        if ( getSqlWhereCondition() != null && !getSqlWhereCondition().isEmpty() )
        {
            return getSqlSelectFromWithJoins() + " WHERE " + getSqlWhereCondition();
        }
        return getSqlSelectFromWithJoins();
    }

    private String getSqlSelectFromWithWhereConditionAndGroupBy()
    {
        if ( groupBy != null && !groupBy.isEmpty() )
        {
            StringBuilder groupingBuilder = new StringBuilder();
            for ( Object groupBy : this.groupBy )
            {
                groupingBuilder.append( "," );
                groupingBuilder.append( groupBy );
            }
            return getSqlSelectFromWithWhereCondition() + " GROUP BY " + groupingBuilder.substring( ",".length() );
        }
        return getSqlSelectFromWithWhereCondition();
    }

    private String getSqlSelectFromWithWhereConditionGroupByAndHavingCondition()
    {
        if ( havingCondition != null && !havingCondition.isEmpty() )
        {
            return getSqlSelectFromWithWhereConditionAndGroupBy() + " HAVING " + getSqlHavingCondition();
        }
        return getSqlSelectFromWithWhereConditionAndGroupBy();
    }

    private String getSqlSelectFromWithWhereConditionGroupByHavingConditionAndOrderBy()
    {
        if ( orderBy != null && !orderBy.isEmpty() )
        {
            StringBuilder orderBuilder = new StringBuilder();
            for ( OrderBy orderBy : this.orderBy )
            {
                orderBuilder.append( "," );
                orderBuilder.append( orderBy.getField() + " " + orderBy.getOrderBy() );
            }
            return getSqlSelectFromWithWhereConditionGroupByAndHavingCondition() + " ORDER BY "
                + orderBuilder.substring( ",".length() );
        }
        return getSqlSelectFromWithWhereConditionGroupByAndHavingCondition();
    }

    public String getAllRowsCounterQueryString()
    {
        return "SELECT count(*) FROM (" + getSqlSelectFromWithWhereConditionGroupByAndHavingCondition()
            + ") AS allRows";
    }

    public String getQueryString()
    {
        if ( startRow >= 0 && rows > 0 )
        {
            return getSqlSelectFromWithWhereConditionGroupByHavingConditionAndOrderBy() + " LIMIT " + startRow + ","
                + rows;
        }
        return getSqlSelectFromWithWhereConditionGroupByHavingConditionAndOrderBy();
    }

}
