/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.windows.freeze;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.translations.TranslatedWords;
import org.dussan.vaadin.dtable.utils.ErrorUtil;

import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.Property.ValueChangeListener;
import com.vaadin.event.FieldEvents.TextChangeEvent;
import com.vaadin.event.FieldEvents.TextChangeListener;
import com.vaadin.server.FontAwesome;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.JavaScript;
import com.vaadin.ui.Label;
import com.vaadin.ui.ListSelect;
import com.vaadin.ui.TextField;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.themes.ValoTheme;

public class FreezeUnfreezeColumns
    extends Window
{

    private static final long serialVersionUID = 3771051434575477411L;

    private int actionDelay = 50;

    private int initDelay = 200;

    private String hiddenColumnDescription = null;

    private List<String> disabledFrozenColumns = null;

    private List<String> disabledUnfrozenColumns = null;

    private List<String> frozenUnfilteredColumns = null;

    private Button closeButton = null;

    private Button freezeButton = null;

    private Button okButton = null;

    private Button unfreezeButton = null;

    private ListSelect frozenColumns = null;

    private ListSelect unfrozenColumns = null;

    private TextField frozenFilter = null;

    private TextField unfrozenFilter = null;

    private DTable dTable = null;

    /**
     * Creates a new instance.
     */
    private FreezeUnfreezeColumns()
    {
        disabledFrozenColumns = new ArrayList<>();
        disabledUnfrozenColumns = new ArrayList<>();
    }

    /**
     * Creates a new instance with dTable object.
     * 
     * @param dTable dTable instance
     */
    public FreezeUnfreezeColumns( DTable dTable )
    {
        this();
        this.dTable = dTable;

        hiddenColumnDescription = " (" + getDTable().getTranslation()
                                                    .get( getDTable().getTableConfig()
                                                                     .getLocale(), TranslatedWords.HIDDEN ) + ")";

        init();
    }

    /**
     * Get dTable instance.
     * 
     * @return dTable instance
     */
    private DTable getDTable()
    {
        return dTable;
    }

    /**
     * Closes window.
     */
    private void closeWindow()
    {
        setVisible( false );
        UI.getCurrent()
          .removeWindow( this );
    }

    /**
     * Javascript code which disable specified columns in list.
     * 
     * @param id lists id
     * @param disabledColumns disabled columns
     * @param delay action delay in ms
     */
    private void setDisabledColumns( String id, String disabledColumns, int delay )
    {
        if ( id != null && !id.isEmpty() && disabledColumns != null && !disabledColumns.isEmpty() && 0 < delay )
        {
            StringBuilder jsCode = new StringBuilder();
            jsCode.append( "setTimeout( function(){" );
            jsCode.append( "    var disabledColumns = " + disabledColumns + ";" );
            jsCode.append( "    var options = document.getElementById('" + id + "').children[0].options;" );
            jsCode.append( "    for( var column = 0; column < disabledColumns.length; column++ ){" );
            jsCode.append( "        for( var option = 0; option < options.length; option++ ){" );
            jsCode.append( "            if( disabledColumns[column] === options[option].text ){" );
            jsCode.append( "                options[option].disabled = true;" );
            jsCode.append( "            }" );
            jsCode.append( "        }" );
            jsCode.append( "    }" );
            jsCode.append( "}, " + delay + " );" );
            JavaScript.getCurrent()
                      .execute( jsCode.toString() );
        }
    }

    /**
     * Get column header index from header value.
     * 
     * @param header header value
     * @return column header index
     */
    private int getColumnHeaderIndex( Object header )
    {
        String headerString = header.toString();
        if ( headerString.endsWith( hiddenColumnDescription ) )
        {
            headerString = headerString.substring( 0, headerString.lastIndexOf( hiddenColumnDescription ) );
        }
        return getDTable().getTableProperties()
                          .getColumnHeaderIndex( headerString );
    }

    /**
     * Get new column size for frozen table.
     * 
     * @return new column size for frozen table
     */
    private int getNewFrozenColumnsSize()
    {
        int columns = 0;
        for ( Object header : frozenColumns.getItemIds() )
        {
            int column = getColumnHeaderIndex( header );
            if ( !getDTable().getTableProperties()
                             .isHiddenColumnFrozen( column ) && !getDTable().getTableProperties()
                                                                            .isRemovedColumn( column ) )
            {
                columns += 1;
            }
        }
        return columns;
    }

    /**
     * Get new column size for table.
     * 
     * @return new column size
     */
    private int getNewUnfrozenColumnsSize()
    {
        int columns = 0;
        for ( Object header : unfrozenColumns.getItemIds() )
        {
            int column = getColumnHeaderIndex( header );
            if ( !getDTable().getTableProperties()
                             .isHiddenColumn( column ) && !getDTable().getTableProperties()
                                                                      .isRemovedColumn( column ) )
            {
                columns += 1;
            }
        }
        return columns;
    }

    /**
     * Add disabled frozen column to list which is later used in javascript code.
     * 
     * @param header column header
     */
    private void addDisabledFrozenColumn( String header )
    {

        if ( header != null && !header.trim()
                                      .isEmpty() )
        {
            disabledFrozenColumns.add( header );
        }
    }

    /**
     * Remove disabled frozen column from list which is later used in javascript code.
     * 
     * @param header column header
     */
    private void removeDisabledFrozenColumn( String header )
    {
        if ( header != null && !header.trim()
                                      .isEmpty() )
        {
            disabledFrozenColumns.remove( header );
        }
    }

    /**
     * Get string from disabled frozen columns array.
     * 
     * @return string from disabled hidden columns array
     */
    private String getDisabledFrozenColumnsArrayString()
    {
        StringBuilder string = new StringBuilder();
        if ( !disabledFrozenColumns.isEmpty() )
        {
            for ( String header : disabledFrozenColumns )
            {
                string.append( ",'" + header + "'" );
            }

            if ( 0 < string.length() )
            {
                string = new StringBuilder( "[" + string.substring( 1 ) + "]" );
            }
        }
        return string.toString();
    }

    /**
     * Add disabled unfrozen column to list which is later used in javascript code.
     * 
     * @param header column header
     */
    private void addDisabledUnfrozenColumn( String header )
    {
        if ( header != null && !header.trim()
                                      .isEmpty() )
        {
            disabledUnfrozenColumns.add( header );
        }
    }

    /**
     * Remove disabled unfrozen column from list which is later used in javascript code.
     * 
     * @param header column header
     */
    private void removeDisabledUnfrozenColumn( String header )
    {
        if ( header != null && !header.trim()
                                      .isEmpty() )
        {
            disabledUnfrozenColumns.remove( header );
        }
    }

    /**
     * Get string from disabled unfrozen columns array.
     * 
     * @return string from disabled unfrozen columns array
     */
    private String getDisabledUnfrozenColumnsArrayString()
    {
        StringBuilder string = new StringBuilder();
        if ( !disabledUnfrozenColumns.isEmpty() )
        {
            for ( String header : disabledUnfrozenColumns )
            {
                string.append( ",'" + header + "'" );
            }

            if ( 0 < string.length() )
            {
                string = new StringBuilder( "[" + string.substring( 1 ) + "]" );
            }
        }
        return string.toString();
    }

    /**
     * Check if button for unfrozen columns may be enabled.
     */
    private void checkUnfreezeButton()
    {
        boolean buttonEnabled = false;
        for ( Object header : frozenColumns.getItemIds() )
        {
            int column = getColumnHeaderIndex( header );
            if ( getDTable().getTableConfig()
                            .canColumnUnfreeze( column ) && !getDTable().getTableProperties()
                                                                        .isRemovedColumn( column ) )
            {
                buttonEnabled = true;
                break;
            }
        }
        unfreezeButton.setEnabled( buttonEnabled );
    }

    /**
     * Check if button for frozen columns may be enabled.
     */
    private void checkFreezeButton()
    {
        boolean buttonEnabled = false;
        for ( Object header : unfrozenColumns.getItemIds() )
        {
            int column = getColumnHeaderIndex( header );
            if ( getDTable().getTableConfig()
                            .canColumnFreeze( column ) && !getDTable().getTableProperties()
                                                                      .isRemovedColumn( column ) )
            {
                buttonEnabled = true;
                break;
            }
        }
        freezeButton.setEnabled( buttonEnabled );
    }

    /**
     * Fill lists with frozen and unfrozen table columns.
     */
    private void fillFrozenAndUnfrozenLists()
    {
        frozenUnfilteredColumns = new ArrayList<>();
        unfrozenColumns.removeAllItems();
        frozenColumns.removeAllItems();
        for ( int column = 0; column < getDTable().getTableProperties()
                                                  .getAllColumns(); column++ )
        {
            String header = getDTable().getTableProperties()
                                       .getColumnHeader( column );

            if ( getDTable().getTableProperties()
                            .isHiddenColumn( column ) )
            {
                header += hiddenColumnDescription;
            }

            if ( ( !getDTable().getTableProperties()
                               .isFrozenColumn( column ) || getDTable().getTableProperties()
                                                                       .isHiddenColumn( column ) )
                && !getDTable().getTableProperties()
                               .isHiddenColumnFrozen( column ) && !getDTable().getTableProperties()
                                                                              .isRemovedColumn( column ) )
            {
                unfrozenColumns.addItem( header );

                if ( !getDTable().getTableConfig()
                                 .canColumnFreeze( column ) )
                {
                    addDisabledUnfrozenColumn( header );
                }
            }

            if ( ( getDTable().getTableProperties()
                              .isFrozenColumn( column ) || getDTable().getTableProperties()
                                                                      .isHiddenColumnFrozen( column ) )
                && !getDTable().getTableProperties()
                               .isRemovedColumn( column ) )
            {
                frozenColumns.addItem( header );
                frozenUnfilteredColumns.add( header );

                if ( !getDTable().getTableConfig()
                                 .canColumnUnfreeze( column ) )
                {
                    addDisabledFrozenColumn( header );
                }
            }
        }

        // disable unfreeze button if frozen table cannot be empty
        checkUnfreezeButton();

        // disable freeze button if unfrozen list is empty
        checkFreezeButton();
    }

    /**
     * Initialization procedure.
     */
    private void init()
    {
        int rows = 10;

        center();
        setClosable( false );
        setDraggable( false );
        setModal( true );
        setResizable( false );
        setWidth( 70, Unit.PERCENTAGE );
        setCaption( getDTable().getTranslation()
                               .get( getDTable().getTableConfig()
                                                .getLocale(), TranslatedWords.FREEZE_UNFREEZE_COLUMNS ) );

        // unfrozen filter
        unfrozenFilter = new TextField( getDTable().getTranslation()
                                                   .get( getDTable().getTableConfig()
                                                                    .getLocale(), TranslatedWords.AVAILABLE_COLUMNS ) );
        unfrozenFilter.addStyleName( ValoTheme.TEXTFIELD_SMALL );
        unfrozenFilter.addTextChangeListener( new TextChangeListener()
        {
            private static final long serialVersionUID = -2825539815904966736L;

            @Override
            public void textChange( TextChangeEvent event )
            {
                String value = event.getText();

                unfrozenColumns.removeAllItems();
                for ( int column = 0; column < getDTable().getTableProperties()
                                                          .getAllColumns(); column++ )
                {
                    String header = getDTable().getTableProperties()
                                               .getColumnHeader( column );

                    if ( getDTable().getTableProperties()
                                    .isHiddenColumn( column ) )
                    {
                        header += hiddenColumnDescription;
                    }

                    if ( ( !getDTable().getTableProperties()
                                       .isFrozenColumn( column ) || getDTable().getTableProperties()
                                                                               .isHiddenColumn( column ) )
                        && !getDTable().getTableProperties()
                                       .isHiddenColumnFrozen( column ) && !getDTable().getTableProperties()
                                                                                      .isRemovedColumn( column )
                        && !frozenColumns.containsId( header ) && ( value.isEmpty() || header.contains( value ) ) )
                    {
                        unfrozenColumns.addItem( header );
                    }
                }

                // disable unfrozen columns which can not freeze
                setDisabledColumns( unfrozenColumns.getId(), getDisabledUnfrozenColumnsArrayString(), actionDelay );

                // disable freeze button if unfrozen list is empty
                checkFreezeButton();
            }
        } );

        // frozen filter
        frozenFilter = new TextField( getDTable().getTranslation()
                                                 .get( getDTable().getTableConfig()
                                                                  .getLocale(), TranslatedWords.FROZEN_COLUMNS ) );
        frozenFilter.addStyleName( ValoTheme.TEXTFIELD_SMALL );
        frozenFilter.addTextChangeListener( new TextChangeListener()
        {
            private static final long serialVersionUID = -2825539815904966736L;

            @Override
            public void textChange( TextChangeEvent event )
            {
                String value = event.getText();

                frozenColumns.removeAllItems();
                for ( String header : frozenUnfilteredColumns )
                {
                    if ( value.isEmpty() || ( !value.isEmpty() && header.contains( value ) ) )
                    {
                        frozenColumns.addItem( header );
                    }
                }

                // disable frozen columns which can not unfreeze
                setDisabledColumns( frozenColumns.getId(), getDisabledFrozenColumnsArrayString(), actionDelay );

                // disable unfreeze button if frozen table cannot be empty
                checkUnfreezeButton();
            }
        } );

        // unfrozen columns
        unfrozenColumns = new ListSelect();
        unfrozenColumns.setId( "unfrozenColumns" );
        unfrozenColumns.setImmediate( true );
        unfrozenColumns.setMultiSelect( true );
        unfrozenColumns.setNullSelectionAllowed( true );
        unfrozenColumns.setRows( rows );
        unfrozenColumns.setWidth( 100, Unit.PERCENTAGE );
        unfrozenColumns.addValueChangeListener( new ValueChangeListener()
        {
            private static final long serialVersionUID = 4691014034226998827L;

            @Override
            public void valueChange( ValueChangeEvent event )
            {
                // disable unfrozen columns which can not freeze
                setDisabledColumns( unfrozenColumns.getId(), getDisabledUnfrozenColumnsArrayString(), actionDelay );
            }
        } );

        // frozen columns
        frozenColumns = new ListSelect();
        frozenColumns.setId( "frozenColumns" );
        frozenColumns.setImmediate( true );
        frozenColumns.setMultiSelect( true );
        frozenColumns.setNullSelectionAllowed( true );
        frozenColumns.setRows( rows );
        frozenColumns.setWidth( 100, Unit.PERCENTAGE );
        frozenColumns.addValueChangeListener( new ValueChangeListener()
        {
            private static final long serialVersionUID = 4691014034226998827L;

            @Override
            public void valueChange( ValueChangeEvent event )
            {
                // disable frozen columns which can not unfreeze
                setDisabledColumns( frozenColumns.getId(), getDisabledFrozenColumnsArrayString(), actionDelay );
            }
        } );

        // unfreeze button
        unfreezeButton = new Button( FontAwesome.CHEVRON_LEFT );
        unfreezeButton.addStyleName( ValoTheme.BUTTON_ICON_ONLY );
        unfreezeButton.addClickListener( new ClickListener()
        {
            private static final long serialVersionUID = -6384761875661773041L;

            @Override
            public void buttonClick( ClickEvent event )
            {
                Object values = frozenColumns.getValue();
                if ( values != null )
                {
                    for ( Object value : (Set<?>) values )
                    {
                        int column = getColumnHeaderIndex( value.toString() );

                        frozenUnfilteredColumns.remove( value );
                        frozenColumns.removeItem( value );
                        if ( getDTable().getTableConfig()
                                        .canColumnUnfreeze( column ) )
                        {
                            unfrozenColumns.addItem( value );
                        }

                        removeDisabledFrozenColumn( value.toString() );
                        if ( !getDTable().getTableConfig()
                                         .canColumnFreeze( column ) )
                        {
                            addDisabledUnfrozenColumn( value.toString() );
                        }
                    }
                }
                frozenColumns.select( null );

                // disable unfrozen and frozen columns in lists
                setDisabledColumns( unfrozenColumns.getId(), getDisabledUnfrozenColumnsArrayString(), actionDelay );
                setDisabledColumns( frozenColumns.getId(), getDisabledFrozenColumnsArrayString(), actionDelay );

                // disable unfreeze button if frozen table cannot be empty
                checkUnfreezeButton();

                // disable freeze button if unfrozen list is empty
                checkFreezeButton();
            }
        } );

        // freeze button
        freezeButton = new Button( FontAwesome.CHEVRON_RIGHT );
        freezeButton.addStyleName( ValoTheme.BUTTON_ICON_ONLY );
        freezeButton.addClickListener( new ClickListener()
        {
            private static final long serialVersionUID = 4097366270472216209L;

            @Override
            public void buttonClick( ClickEvent event )
            {
                Object values = unfrozenColumns.getValue();
                if ( values != null )
                {
                    for ( Object value : (Set<?>) values )
                    {
                        int column = getDTable().getTableProperties()
                                                .getColumnHeaderIndex( value.toString() );

                        unfrozenColumns.removeItem( value );
                        if ( getDTable().getTableConfig()
                                        .canColumnFreeze( column ) )
                        {
                            frozenUnfilteredColumns.add( value.toString() );
                            frozenColumns.addItem( value );

                            removeDisabledUnfrozenColumn( value.toString() );
                            if ( !getDTable().getTableConfig()
                                             .canColumnUnfreeze( column ) )
                            {
                                addDisabledFrozenColumn( value.toString() );
                            }
                        }
                    }
                }
                unfrozenColumns.select( null );

                // disable unfrozen and frozen columns in lists
                setDisabledColumns( unfrozenColumns.getId(), getDisabledUnfrozenColumnsArrayString(), actionDelay );
                setDisabledColumns( frozenColumns.getId(), getDisabledFrozenColumnsArrayString(), actionDelay );

                // disable unfreeze button if frozen table cannot be empty
                checkUnfreezeButton();

                // disable freeze button if unfrozen list is empty
                checkFreezeButton();
            }
        } );

        // "OK" button
        okButton = new Button( getDTable().getTranslation()
                                          .get( getDTable().getTableConfig()
                                                           .getLocale(), TranslatedWords.OK ) );
        okButton.addClickListener( new ClickListener()
        {
            private static final long serialVersionUID = -5941114122010602045L;

            @Override
            public void buttonClick( ClickEvent event )
            {
                List<Integer> columns = new ArrayList<>();
                if ( frozenColumns.getItemIds() != null )
                {
                    for ( Object header : (Collection<?>) frozenColumns.getItemIds() )
                    {
                        columns.add( getColumnHeaderIndex( header ) );
                    }
                }
                Collections.sort( columns );

                if ( getNewUnfrozenColumnsSize() < 1 )
                {
                    new ErrorUtil(
                                   getDTable().getTranslation()
                                              .get( getDTable().getTableConfig()
                                                               .getLocale(), TranslatedWords.ERROR ),
                                   getDTable().getTranslation()
                                              .get( getDTable().getTableConfig()
                                                               .getLocale(),
                                                    TranslatedWords.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_UNFROZEN_AND_UNHIDDEN ) ).show();
                }
                else if ( !getDTable().getTableConfig()
                                      .isEmptyFrozenTableEnabled() && getNewFrozenColumnsSize() < 1 )
                {
                    new ErrorUtil( getDTable().getTranslation()
                                              .get( getDTable().getTableConfig()
                                                               .getLocale(), TranslatedWords.ERROR ),
                                   getDTable().getTranslation()
                                              .get( getDTable().getTableConfig()
                                                               .getLocale(),
                                                    TranslatedWords.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_FROZEN_AND_UNHIDDEN ) ).show();
                }
                else if ( getDTable().getTableConfig()
                                     .getTableFrozenMaximumColumns() < getNewFrozenColumnsSize() )
                {
                    new ErrorUtil( getDTable().getTranslation()
                                              .get( getDTable().getTableConfig()
                                                               .getLocale(), TranslatedWords.ERROR ),
                                   getDTable().getTranslation()
                                              .get( getDTable().getTableConfig()
                                                               .getLocale(),
                                                    TranslatedWords.MAXIMUM_PERMITTED_NUMBER_OF_FROZEN_COLUMNS_IS )
                                       + getDTable().getTableConfig()
                                                    .getTableFrozenMaximumColumns() ).show();
                }
                else
                {
                    // set/remove frozen columns
                    for ( Integer column = 0; column < getDTable().getTableProperties()
                                                                  .getAllColumns(); column++ )
                    {
                        if ( columns.contains( column ) && getDTable().getTableConfig()
                                                                      .canColumnFreeze( column )
                            && !getDTable().getTableProperties()
                                           .isFrozenColumn( column ) && !getDTable().getTableProperties()
                                                                                    .isHiddenColumnFrozen( column )
                            && !getDTable().getTableProperties()
                                           .isRemovedColumn( column ) )
                        {
                            getDTable().getTableProperties()
                                       .setFrozenColumn( column );
                        }
                        else if ( !columns.contains( column ) && getDTable().getTableConfig()
                                                                            .canColumnUnfreeze( column )
                            && !getDTable().getTableProperties()
                                           .isRemovedColumn( column )
                            && ( getDTable().getTableProperties()
                                            .isFrozenColumn( column ) || getDTable().getTableProperties()
                                                                                    .isHiddenColumnFrozen( column ) ) )
                        {
                            getDTable().getTableProperties()
                                       .removeFrozenColumn( column );
                        }
                    }

                    // close window
                    closeWindow();

                    // set new table width
                    getDTable().getTableProperties()
                               .calculateTableWidth( getDTable().getTableConfig()
                                                                .getColumnsWidth() );

                    // refresh visible data
                    getDTable().refreshVisibleData( "freezeUnfreezeColumns", true );
                }
            }
        } );

        // close button
        closeButton =
            new Button( getDTable().getTranslation()
                                   .get( getDTable().getTableConfig()
                                                    .getLocale(), TranslatedWords.CLOSE ), new ClickListener()
            {
                private static final long serialVersionUID = 5390169876653829863L;

                @Override
                public void buttonClick( ClickEvent event )
                {
                    closeWindow();
                }
            } );

        // explanation label
        Label explanationLabel =
            new Label( getDTable().getTranslation()
                                  .get( getDTable().getTableConfig()
                                                   .getLocale(),
                                        TranslatedWords.COLUMNS_THAT_CAN_NOT_BE_FROZEN_OR_UNFROZEN_ARE_DISABLED ) );

        // layouts
        VerticalLayout frozenLayout = new VerticalLayout( frozenFilter, frozenColumns );
        frozenLayout.setMargin( false );
        frozenLayout.setSpacing( true );
        frozenFilter.setWidth( 100, Unit.PERCENTAGE );

        VerticalLayout unfrozenLayout = new VerticalLayout( unfrozenFilter, unfrozenColumns );
        unfrozenLayout.setMargin( false );
        unfrozenLayout.setSpacing( true );
        unfrozenFilter.setWidth( 100, Unit.PERCENTAGE );

        VerticalLayout actionLayout = new VerticalLayout( freezeButton, unfreezeButton );
        actionLayout.setMargin( false );
        actionLayout.setSizeUndefined();
        actionLayout.setSpacing( false );

        HorizontalLayout columnsLayout = new HorizontalLayout( unfrozenLayout, actionLayout, frozenLayout );
        columnsLayout.setComponentAlignment( actionLayout, Alignment.MIDDLE_CENTER );
        columnsLayout.setExpandRatio( unfrozenLayout, 1.0f );
        columnsLayout.setExpandRatio( frozenLayout, 1.0f );
        columnsLayout.setMargin( false );
        columnsLayout.setSpacing( true );
        columnsLayout.setWidth( 100, Unit.PERCENTAGE );

        VerticalLayout mainLayout = new VerticalLayout( columnsLayout, explanationLabel );
        mainLayout.setMargin( true );
        mainLayout.setSpacing( true );
        mainLayout.setWidth( 100, Unit.PERCENTAGE );
        mainLayout.addStyleName( "context" );

        HorizontalLayout buttonsLayout = new HorizontalLayout( okButton, closeButton );
        buttonsLayout.setComponentAlignment( okButton, Alignment.MIDDLE_RIGHT );
        buttonsLayout.setComponentAlignment( closeButton, Alignment.MIDDLE_LEFT );
        buttonsLayout.setMargin( true );
        buttonsLayout.setSpacing( true );

        VerticalLayout windowLayout = new VerticalLayout( mainLayout, buttonsLayout );
        windowLayout.setComponentAlignment( buttonsLayout, Alignment.BOTTOM_CENTER );
        windowLayout.setMargin( false );
        windowLayout.setSpacing( false );
        windowLayout.addStyleName( "v-dtable-dialog" );
        setContent( windowLayout );

        // init fill frozen and unfrozen lists
        fillFrozenAndUnfrozenLists();

        // disable unfrozen and frozen columns in lists
        setDisabledColumns( unfrozenColumns.getId(), getDisabledUnfrozenColumnsArrayString(), initDelay );
        setDisabledColumns( frozenColumns.getId(), getDisabledFrozenColumnsArrayString(), initDelay );
    }

}
